#!/bin/bash

AWK="/usr/bin/awk"
ECHO="/bin/echo"
GREP="/usr/bin/grep"
ID="/usr/xpg4/bin/id"
LOGINS="/usr/bin/logins"
LS="/bin/ls"
PASSWD="/usr/bin/passwd"
SED="/usr/bin/sed"
SVCADM="/usr/sbin/svcadm"
SVCCFG="/usr/sbin/svccfg"
SVCS="/usr/bin/svcs"
UNAME="/bin/uname"
WC="/bin/wc"

OS="unknown"
OS_VERSION="unknown"

SUCCESS=0
ERROR=1

function print_error {
   ${ECHO} "Error: $@" >&2
}

function print_warn {
   ${ECHO} "WARNING: $@"
}

function print_info {
   ${ECHO} "INFO: $@"
}

function print_ok {
   ${ECHO} "OK: $@"
}

function print_banner {
   ${ECHO} "[ $@ ]"
}

function print_divider {
   ${ECHO} "==========================================================================================="
}

function check_os {
   OPERATING_SYSTEM=$( ${UNAME} -s )   
   OS_REVISION=$( ${UNAME} -r )
   case "${OPERATING_SYSTEM}" in
      "SunOS")	OS="Solaris"
		case "${OS_REVISION}" in
		   "5.10")  OS_VERSION="10" ;;
		   "5.9" )  OS_VERSION="9"  ;;
		   "5.8" )  OS_VERSION="8"  ;;
                   "5.7" )  OS_VERSION="7"  ;;
                   *     )  OS_VERSION="UNSUPPORTED" ;;
                esac ;;
       *     )  OS="UNSUPPORTED" ;;
   esac
   if [ "${OS}" = "UNSUPPORTED" -o "${OS_VERSION}" = "UNSUPPORTED" ]; then
      print_error "Sorry, ${OPERATING_SYSTEM} ${OS_REVISION} is not supported"
      exit ${ERROR}
   elif [ "${OS_VERSION}" -ne "10" ]; then   # temporary....
      print_error "Sorry, only Solaris 10 supported at present"
      exit ${ERROR}
   fi
}

function check_user {
   MY_UID=$( ${ID} -u )
   if [ "${MY_UID}" -ne "0" ]; then
      print_error "This script must be executed as root"
      exit ${ERROR}
   fi
}

function check_system_accounts {
   print_banner "Checking System Accounts"
   print_divider
   case ${OS_VERSION} in
      "10")
          USERLIST="bin nuucp smmsp listen gdm webserverd nobody noaccess nobody4"
          for SYSUSER in ${USERLIST}; do
             ${PASSWD} -s ${SYSUSER} >/dev/null 2>&1
             if [ "$?" -eq "0" ]; then
                PASSWD_STATE=$( ${PASSWD} -s ${SYSUSER} | ${AWK} '{print $NF}' )
                PASSWD_ERR_COUNT=0
                if [ "${PASSWD_STATE}" != "LK" ]; then
                   print_warn "Account ${SYSUSER}'s password is state [${PASSWD_STATE}]"
                   print_info "Consider locking with \"passwd -l ${SYSUSER}\""   
                   (( PASSWD_ERR_COUNT = PASSWD_ERR_COUNT + 1 ))
                fi
                SHELL_VALUE=$( ${LOGINS} -xl ${SYSUSER} | ${SED} -n '3p' | ${AWK} '{print $1}' ) 
                if [ "${SHELL_VALUE}" != "/dev/null" -o "${SHELL_VALUE}" != "/bin/false" ]; then
                   print_warn "Account ${SYSUSER}'s shell is [${SHELL_VALUE}]"
                   print_info "Consider changing to /dev/null or /bin/false with:"
                   print_info "passmgmt -m -s /dev/null ${SYSUSER}"
                   (( PASSWD_ERR_COUNT = PASSWD_ERR_COUNT + 1 ))
                fi
                if [ "${PASSWD_ERR_COUNT}" -eq "0" ]; then
                   print_ok "System user ${SYSUSER} restricted OK"
                fi 
             else
                print_ok "System user ${SYSUSER} not present"
             fi
          done
          USERLIST="sys adm lp uucp"
          for SYSUSER in ${USERLIST}; do
             ${PASSWD} -s ${SYSUSER} >/dev/null 2>&1
             if [ "$?" -eq "0" ]; then
                PASSWD_STATE=$( ${PASSWD} -s ${SYSUSER} | ${AWK} '{print $NF}' )
                PASSWD_ERR_COUNT=0
                if [ "${PASSWD_STATE}" != "NL" -a "${PASSWD_STATE}" != "LK" ]; then
                   print_warn "Account ${SYSUSER}'s password is state [${PASSWD_STATE}]"
                   print_info "Consider locking with \"passwd -N ${SYSUSER}\""   
                   (( PASSWD_ERR_COUNT = PASSWD_ERR_COUNT + 1 ))
                fi
                SHELL_VALUE=$( ${LOGINS} -xl ${SYSUSER} | ${SED} -n '3p' | ${AWK} '{print $1}' ) 
                if [ "${SHELL_VALUE}" != "/dev/null" -o "${SHELL_VALUE}" != "/bin/false" ]; then
                   print_warn "Account ${SYSUSER}'s shell is [${SHELL_VALUE}]"
                   print_info "Consider changing to /dev/null or /bin/false with:"
                   print_info "passmgmt -m -s /dev/null ${SYSUSER}"
                   (( PASSWD_ERR_COUNT = PASSWD_ERR_COUNT + 1 ))
                fi
                if [ "${PASSWD_ERR_COUNT}" -eq "0" ]; then
                   print_ok "System user ${SYSUSER} restricted OK"
                fi 
             else
                print_ok "System user ${SYSUSER} not present"
             fi
          done
          ;;
   esac
}

function check_empty_passwds {
   print_banner "Checking for Accounts With Empty Passwords"
   print_divider
   case ${OS_VERSION} in
     "10")
         OUTPUT=$( ${LOGINS} -p )
         OUTPUT_LINES=$( ${ECHO} "${OUTPUT}" | ${WC} -l )
         if [ "${OUTPUT_LINES}" -ne "0" -a -n "${OUTPUT}" ]; then
            print_warn "${OUTPUT_LINES} account(s) has/have an empty password"
            print_info "Fix this immediately with the following commands:"
            ${ECHO} "${OUTPUT}" | ${AWK} '{print $1}' | while read ACCOUNT; do
               print_info "passwd -l ${ACCOUNT}"
            done
         else
            print_ok "No accounts have empty passwords"
         fi
         ;;
   esac
}

function check_legacy_entries {
   print_banner "Checking for Legacy \"+\" Entries in passwd Files"
   print_divider
   case ${OS_VERSION} in
      "10")
          PASSWD_FILES="/etc/passwd /etc/group /etc/shadow"
          PASSWD_ERR_COUNT=0
          for PASSWD_FILE in ${PASSWD_FILES}; do
             ${GREP} '^+:' ${PASSWD_FILE} >/dev/null 2>&1
             if [ "$?" -eq "0" ]; then
                print_warn "\"+\" entry found in ${PASSWD_FILE}"
                (( PASSWD_ERR_COUNT = PASSWD_ERR_COUNT + 1 ))
             fi
          done
          if [ "$?" -eq "0" ]; then
             print_ok "No legacy \"+\" entries found"
          fi
          ;;
   esac
}

function check_uid_zero {
   print_banner "Checking for UID 0 Accounts"
   print_divider
   case ${OS_VERSION} in
     "10")
         UID_0_ACCOUNTS=$( ${LOGINS} -o | ${AWK} -F: '$2==0 {print $1}' )
         if [ "${UID_0_ACCOUNTS}" != "root" ]; then
            print_warn "More than one UID 0 account found"
            ${ECHO} "${UID_0_ACCOUNTS}" | while read UID_0; do
               print_info "${UID_0}"
            done
         else
            print_ok "Only \"root\" has UID 0"
         fi
         ;;
   esac
}

function check_umask {
   print_banner "Checking UMASK and mesg Settings"
   print_divider
   case ${OS_VERSION} in
     "10")
         ${GREP} "^UMASK=077" /etc/default/login >/dev/null 2>&1
         if [ "$?" -ne "0" ]; then
            print_warn "UMASK should be more restrictive in /etc/default/login"
            print_info "Execute the following commands to implement this:"
            print_info "cd /etc/default"
            print_info "sed 's/^[#]\{0,1\}UMASK=.*$/UMASK=077/' login > login.new"
            print_info "mv login.new login"
            print_info "pkgchk -f -n -p /etc/default/login"
         else
            print_ok "UMASK set correctly in /etc/default/login"
         fi
         PROFILE_FILES="/etc/profile /etc/.login"
         for PROFILE in ${PROFILE_FILES}; do
            if [ -e "${PROFILE}" ]; then
               ${GREP} "^[^#]*umask 077" ${PROFILE} >/dev/null 2>&1
               if [ "$?" -ne "0" ]; then
                  print_warn "umask not set correctly in ${PROFILE}"
               else
                  print_ok "umask set correctly in ${PROFILE}"
               fi
               ${GREP} "^[^#]*mesg n" ${PROFILE} >/dev/null 2>&1
               if [ "$?" -ne "0" ]; then
                  print_warn "mesg not set correctly in ${PROFILE}"
               else
                  print_ok "mesg set correctly in ${PROFILE}"
               fi
            fi
         done
         ;;
   esac
}

check_os
check_user
print_divider
check_system_accounts
print_divider
check_empty_passwds
print_divider
# Don't bother with password aging/complexity requirements.. yet..
check_legacy_entries
print_divider
check_uid_zero
print_divider
check_umask
print_divider


exit 0
